/** @file
 *
 * @defgroup ble_lbs LED Button Service Server
 * @{
 * @ingroup ble_sdk_srv
 *
 * @brief LED Button Service Server module.
 *
 * @details This module implements a custom LED Button Service with an LED and Button Characteristics.
 *          During initialization, the module adds the LED Button Service and Characteristics
 *          to the BLE stack database.
 *
 *          The application must supply an event handler for receiving LED Button Service
 *          events. Using this handler, the service notifies the application when the
 *          LED value changes.
 *
 *          The service also provides a function for letting the application notify
 *          the state of the Button Characteristic to connected peers.
 *
 * @note    The application must register this module as BLE event observer using the
 *          NRF_SDH_BLE_OBSERVER macro. Example:
 *          @code
 *              ble_hids_t instance;
 *              NRF_SDH_BLE_OBSERVER(anything, BLE_HIDS_BLE_OBSERVER_PRIO,
 *                                   ble_hids_on_ble_evt, &instance);
 *          @endcode
 */

#ifndef BLE_TGOI_H__
#define BLE_TGOI_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"
#include "nrf_sdh_ble.h"

#ifdef __cplusplus
extern "C" {
#endif

/**@brief   Macro for defining a ble_tgoi instance.
 *
 * @param   _name   Name of the instance.
 * @hideinitializer
 */
#define BLE_TGOI_DEF(_name)                                                                          \
static ble_tgoi_t _name;                                                                             \
NRF_SDH_BLE_OBSERVER(_name ## _obs,                                                                 \
                     BLE_TGOI_BLE_OBSERVER_PRIO,                                                     \
                     ble_tgoi_on_ble_evt, &_name)

#define TGOI_UUID_BASE       {0x57,0x49,0x4C,0x44,0x44,0x52,0x41,0x47,0x4F,0x4E,0x43,0x4C,0x00,0x01,0x50,0xFF}
#define TGOI_UUID_SERVICE     0x4499
#define TGOI_UUID_SOMI_CHAR   (TGOI_UUID_SERVICE + 1)
#define TGOI_UUID_SIMO_CHAR   (TGOI_UUID_SERVICE + 2)

															
// Forward declaration of the ble_tgoi_t type.
typedef struct ble_tgoi_s ble_tgoi_t;

typedef void (*ble_tgoi_SIMO_write_handler_t) (uint16_t conn_handle, ble_tgoi_t * p_tgoi, uint8_t* data,uint16_t dataLen);

/** @brief LED Button Service init structure. This structure contains all options and data needed for
 *        initialization of the service.*/
typedef struct
{
    ble_tgoi_SIMO_write_handler_t SIMO_write_handler; /**< Event handler to be called when the LED Characteristic is written. */
} ble_tgoi_init_t;

/**@brief TGOI Service structure. This structure contains various status information for the service. */
struct ble_tgoi_s
{
    uint16_t                    service_handle;      /**< Handle of LED Button Service (as provided by the BLE stack). */
    ble_gatts_char_handles_t    SIMO_char_handles;    /**< Handles related to the LED Characteristic. */
    ble_gatts_char_handles_t    SOMI_char_handles; /**< Handles related to the Button Characteristic. */
    uint8_t                     uuid_type;           /**< UUID type for the LED Button Service. */
    ble_tgoi_SIMO_write_handler_t SIMO_write_handler;   /**< Event handler to be called when the LED Characteristic is written. */
};


/**@brief Function for initializing the LED Button Service.
 *
 * @param[out] p_lbs      LED Button Service structure. This structure must be supplied by
 *                        the application. It is initialized by this function and will later
 *                        be used to identify this particular service instance.
 * @param[in] p_tgoi_init  Information needed to initialize the service.
 *
 * @retval NRF_SUCCESS If the service was initialized successfully. Otherwise, an error code is returned.
 */
uint32_t ble_tgoi_init(ble_tgoi_t * p_tgoi, const ble_tgoi_init_t * p_tgoi_init);


/**@brief Function for handling the application's BLE stack events.
 *
 * @details This function handles all events from the BLE stack that are of interest to the LED Button Service.
 *
 * @param[in] p_ble_evt  Event received from the BLE stack.
 * @param[in] p_context  LED Button Service structure.
 */
void ble_tgoi_on_ble_evt(ble_evt_t const * p_ble_evt, void * p_context);


/**@brief Function for sending a button state notification.
 *
 ' @param[in] conn_handle   Handle of the peripheral connection to which the button state notification will be sent.
 * @param[in] p_lbs         LED Button Service structure.
 * @param[in] button_state  New button state.
 *
 * @retval NRF_SUCCESS If the notification was sent successfully. Otherwise, an error code is returned.
 */
uint32_t ble_tgoi_on_SOMI_change(uint16_t conn_handle, ble_tgoi_t * p_tgoi, uint8_t* data,uint16_t dataLen);


#ifdef __cplusplus
}
#endif

#endif // BLE_LBS_H__

/** @} */
